// STB_mobo 0.0.2 2010:08:18 00:00:00$
// Copyright (c) 2010 stb
// Released under the GNU General Public License
//
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// Based on Bollinger_TripleState_v4NT7 
    /// Bollinger_TripleState_v4NT7 Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
    /// STB_mobo simulates the appearence from http://fibonacci-financial.blogspot.com/2010/03/david-elliott-what-are-these-charts.html
    /// STB_mobo identifies momemtum break outs
    /// it sets the defaults indicated by gio5959 at http://www.mypivots.com/
	/// </summary>
	[Description("Bollinger MOBO")]
	public class STB_mobo : Indicator
	{
		#region Variables
		private	double		numStdDev	= 0.8;     
        private int         period = 10;       
        private double wide_Tick_Threashold = 8;
        private double narrow_Tick_Threashold = 4;

        private Color sMALineColor = Color.Gray;
		private bool 		myLines = true;
        private bool shading = true;

        private Color stb_upClose_Color = Color.Green;       
        private Color stb_downClose_Color = Color.Red;
        private Color stb_neutralClose_Color = Color.Gray;

        private Color stb_upBand_Color = Color.Gray;       
        private Color stb_downBand_Color = Color.Gray;     
        private Color stb_middelBand_Color = Color.Gray;     
        private bool markArrows = true;
        int arrowMarked = 0;    // 0 neutral, +1 up, -1 down
        private bool drawText = false;

		private DataSeries  value1;
		private DataSeries  value2;
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
            Add(new Plot(stb_upBand_Color, PlotStyle.Line, "Upper"));         // 0
            Add(new Plot(stb_downBand_Color, PlotStyle.Line, "Lower"));          // 1
            Add(new Plot(stb_middelBand_Color, PlotStyle.Line, "Middle"));        // 2
			
			Plots[0].Pen.Width = 1;
			Plots[1].Pen.Width = 1;
			Plots[2].Pen.Width = 1;
			
			value1	= new DataSeries(this);
			value2	= new DataSeries(this);
			
			Overlay				= true;
			PriceTypeSupported	= true;
			
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if(CurrentBar < Period) return;
			
			double line_upper = SMA(Period)[0] + NumStdDev * StdDev(Period)[0];//get upper line value
			double line_lower = SMA(Period)[0] - NumStdDev * StdDev(Period)[0];//get lower line value
			
			Upper.Set(line_upper);//set the ds values to lines above
			Lower.Set(line_lower);

            double diff = Upper[0] - Lower[0];//calc diff btw line

            double wtt = wide_Tick_Threashold * TickSize;
            double ntt = narrow_Tick_Threashold * TickSize;
			
            if (drawText) DrawTextFixed("Threashold Determiner", "Actual Upper/Lower Band Spread: " + Math.Round(diff, 4), TextPosition.TopLeft);  // _e_

            // _e_
            if (Closes[0][0] > Upper[0])
            {
                stb_upBand_Color = stb_upClose_Color;
                stb_downBand_Color = stb_upClose_Color;
                 if (arrowMarked != 1)
                {
                    if (markArrows) DrawArrowUp("My up arrow" + CurrentBar, false, 0, Closes[0][0] - (0 * TickSize), stb_upClose_Color);
                     arrowMarked = 1;
                }
            }
            if (Closes[0][0] < Lower[0])
            {
                stb_upBand_Color = stb_downClose_Color;
                stb_downBand_Color = stb_downClose_Color;
                if (arrowMarked != -1)
                {
                    if (markArrows) DrawArrowDown("My down arrow" + CurrentBar, false, 0, Closes[0][0] + (0 * TickSize), stb_downClose_Color);
                    arrowMarked = -1;
                }
            }

            if (shading)
            {
                if (arrowMarked == 1)
                {
                    DrawRegion("reg1" + CurrentBar, 0, 1, Upper, Lower, Color.Empty, stb_upClose_Color, 3);
                }
                else
                {
                    DrawRegion("reg2" + CurrentBar, 0, 1, Upper, Lower, Color.Empty, stb_downClose_Color, 3);
                }
            }

            if (myLines)
            {
                PlotColors[0][0] = stb_upBand_Color;
                PlotColors[1][0] = stb_downBand_Color;	
            }
            else
            {
                stb_upBand_Color = Color.Transparent;
                stb_downBand_Color = Color.Transparent;
            }
        }

		#region Properties
        [Description("Do you want text shown")]
        [Category("Parameters")]
        public bool DrawText
        {
            get { return drawText; }
            set { drawText = value; }
        }

        [Description("Do you want shading")]
        [Category("Parameters")]
        public bool Shading
        {
            get { return shading; }
            set { shading = value; }
        }

        [Description("Do you want to draw the lines")]
        [Category("Parameters")]
        public bool MyLines
        {
            get { return myLines; }
            set { myLines = value; }
        }
        [Description("Do you want to show arrows")]
        [Category("Parameters")]
        public bool MarkArrows
        {
            get { return markArrows; }
            set { markArrows = value; }
        }
        [Description("threashold to draw narrow color")]
        [Category("Parameters")]
        public double Wide_Threashold
        {
            get { return wide_Tick_Threashold; }
            set { wide_Tick_Threashold = Math.Max(0.001, value); }
        }

        [Description("threashold to draw narrow color")]
        [Category("Parameters")]
        public double Narrow_Tick_Threashold
        {
            get { return narrow_Tick_Threashold; }
            set { narrow_Tick_Threashold = Math.Max(0.001, value); }
        }
		[XmlIgnore()]
        [Description("UpClose_Color")]
        [Category("Colors")]
        public Color STB_UpClose_Color
        {
            get { return stb_upClose_Color; }
            set { stb_upClose_Color = value; }
        }
				[Browsable(false)]
        public string STB_UpClose_ColorSerialize
		{
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(STB_UpClose_Color); }
            set { STB_UpClose_Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("DownClose_Color")]
        [Category("Colors")]
                public Color STB_DownClose_Color
        {
            get { return stb_downClose_Color; }
            set { stb_downClose_Color = value; }
        }
						[Browsable(false)]
        public string STB_DownClose_ColorSerialize
		{
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(STB_DownClose_Color); }
            set { STB_DownClose_Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

				[XmlIgnore()]
                [Description("NeutralClose_Color")]
        [Category("Colors")]
        public Color STB_NeutralClose_Color
        {
            get { return stb_neutralClose_Color; }
            set { stb_neutralClose_Color = value; }
        }
						[Browsable(false)]
        public string STB_NeutralClose_ColorSerialize
		{
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(STB_NeutralClose_Color); }
            set { STB_NeutralClose_Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
						[XmlIgnore()]
        [Description("SMA Line Color")]
        [Category("Colors")]
        public Color SMALineColor
        {
            get { return sMALineColor; }
            set { sMALineColor = value; }
        }
						[Browsable(false)]
		public string SMALineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(SMALineColor); }
			set { SMALineColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		/// <summary>
		/// Gets the lower value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Lower
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// Get the upper value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Upper
		{
			get { return Values[1]; }
		}
		/// <summary>
		/// Get the middle value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Middle
		{
			get { return Values[2]; }
		}

		/// <summary>
		/// </summary>
		[Description("Number of standard deviations")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("# of std. dev.")]
		public double NumStdDev
		{
			get { return numStdDev; }
			set { numStdDev = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private STB_mobo[] cacheSTB_mobo = null;

        private static STB_mobo checkSTB_mobo = new STB_mobo();

        /// <summary>
        /// Bollinger_TripleState_v4NT7 Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public STB_mobo STB_mobo(bool myLines, double narrow_Tick_Threashold, double numStdDev, int period, bool shading, double wide_Threashold)
        {
            return STB_mobo(Input, myLines, narrow_Tick_Threashold, numStdDev, period, shading, wide_Threashold);
        }

        /// <summary>
        /// Bollinger_TripleState_v4NT7 Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public STB_mobo STB_mobo(Data.IDataSeries input, bool myLines, double narrow_Tick_Threashold, double numStdDev, int period, bool shading, double wide_Threashold)
        {
            if (cacheSTB_mobo != null)
                for (int idx = 0; idx < cacheSTB_mobo.Length; idx++)
                    if (cacheSTB_mobo[idx].MyLines == myLines && Math.Abs(cacheSTB_mobo[idx].Narrow_Tick_Threashold - narrow_Tick_Threashold) <= double.Epsilon && Math.Abs(cacheSTB_mobo[idx].NumStdDev - numStdDev) <= double.Epsilon && cacheSTB_mobo[idx].Period == period && cacheSTB_mobo[idx].Shading == shading && Math.Abs(cacheSTB_mobo[idx].Wide_Threashold - wide_Threashold) <= double.Epsilon && cacheSTB_mobo[idx].EqualsInput(input))
                        return cacheSTB_mobo[idx];

            lock (checkSTB_mobo)
            {
                checkSTB_mobo.MyLines = myLines;
                myLines = checkSTB_mobo.MyLines;
                checkSTB_mobo.Narrow_Tick_Threashold = narrow_Tick_Threashold;
                narrow_Tick_Threashold = checkSTB_mobo.Narrow_Tick_Threashold;
                checkSTB_mobo.NumStdDev = numStdDev;
                numStdDev = checkSTB_mobo.NumStdDev;
                checkSTB_mobo.Period = period;
                period = checkSTB_mobo.Period;
                checkSTB_mobo.Shading = shading;
                shading = checkSTB_mobo.Shading;
                checkSTB_mobo.Wide_Threashold = wide_Threashold;
                wide_Threashold = checkSTB_mobo.Wide_Threashold;

                if (cacheSTB_mobo != null)
                    for (int idx = 0; idx < cacheSTB_mobo.Length; idx++)
                        if (cacheSTB_mobo[idx].MyLines == myLines && Math.Abs(cacheSTB_mobo[idx].Narrow_Tick_Threashold - narrow_Tick_Threashold) <= double.Epsilon && Math.Abs(cacheSTB_mobo[idx].NumStdDev - numStdDev) <= double.Epsilon && cacheSTB_mobo[idx].Period == period && cacheSTB_mobo[idx].Shading == shading && Math.Abs(cacheSTB_mobo[idx].Wide_Threashold - wide_Threashold) <= double.Epsilon && cacheSTB_mobo[idx].EqualsInput(input))
                            return cacheSTB_mobo[idx];

                STB_mobo indicator = new STB_mobo();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.MyLines = myLines;
                indicator.Narrow_Tick_Threashold = narrow_Tick_Threashold;
                indicator.NumStdDev = numStdDev;
                indicator.Period = period;
                indicator.Shading = shading;
                indicator.Wide_Threashold = wide_Threashold;
                Indicators.Add(indicator);
                indicator.SetUp();

                STB_mobo[] tmp = new STB_mobo[cacheSTB_mobo == null ? 1 : cacheSTB_mobo.Length + 1];
                if (cacheSTB_mobo != null)
                    cacheSTB_mobo.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSTB_mobo = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Bollinger_TripleState_v4NT7 Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_mobo STB_mobo(bool myLines, double narrow_Tick_Threashold, double numStdDev, int period, bool shading, double wide_Threashold)
        {
            return _indicator.STB_mobo(Input, myLines, narrow_Tick_Threashold, numStdDev, period, shading, wide_Threashold);
        }

        /// <summary>
        /// Bollinger_TripleState_v4NT7 Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_mobo STB_mobo(Data.IDataSeries input, bool myLines, double narrow_Tick_Threashold, double numStdDev, int period, bool shading, double wide_Threashold)
        {
            return _indicator.STB_mobo(input, myLines, narrow_Tick_Threashold, numStdDev, period, shading, wide_Threashold);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Bollinger_TripleState_v4NT7 Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_mobo STB_mobo(bool myLines, double narrow_Tick_Threashold, double numStdDev, int period, bool shading, double wide_Threashold)
        {
            return _indicator.STB_mobo(Input, myLines, narrow_Tick_Threashold, numStdDev, period, shading, wide_Threashold);
        }

        /// <summary>
        /// Bollinger_TripleState_v4NT7 Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_mobo STB_mobo(Data.IDataSeries input, bool myLines, double narrow_Tick_Threashold, double numStdDev, int period, bool shading, double wide_Threashold)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.STB_mobo(input, myLines, narrow_Tick_Threashold, numStdDev, period, shading, wide_Threashold);
        }
    }
}
#endregion
